@@ -24,7 +24,6 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.
 """
 
-import time
 import asyncio
 
 import discord.abc
@@ -33,7 +32,7 @@ from .enums import ChannelType, try_enum, VoiceRegion
 from .mixins import Hashable
 from . import utils
 from .asset import Asset
-from .errors import ClientException, NoMoreItems, InvalidArgument
+from .errors import ClientException, NoMoreItems, InvalidArgument, NotFound
 
 __all__ = (
     'TextChannel',
@@ -46,9 +45,13 @@ __all__ = (
     '_channel_factory',
 )
 
-async def _single_delete_strategy(messages):
-    for m in messages:
-        await m.delete()
+async def _delete_messages(state, channel_id, messages):
+    delete_message = state.http.delete_message
+    for msg in messages:
+        try:
+            await delete_message(channel_id, msg.id)
+        except NotFound:
+            pass
 
 class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
     """Represents a Discord guild text channel.
@@ -174,8 +177,9 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
             :class: helpful
 
             For a slightly more reliable method of fetching the
-            last message, consider using :meth:`history`
-            with the :attr:`last_message_id` attribute.
+            last message, consider using either :meth:`history`
+            or :meth:`fetch_message` with the :attr:`last_message_id`
+            attribute.
 
         Returns
         ---------
@@ -184,7 +188,7 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
         """
         return self._state._get_message(self.last_message_id) if self.last_message_id else None
 
-    async def edit(self, *, reason=None, **options):
+    async def edit(self, **options):
         """|coro|
 
         Edits the channel.
@@ -221,8 +225,6 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
             Change the type of this text channel. Currently, only conversion between
             :attr:`ChannelType.text` and :attr:`ChannelType.news` is supported. This
             is only available to guilds that contain ``NEWS`` in :attr:`Guild.features`.
-        reason: Optional[:class:`str`]
-            The reason for editing this channel. Shows up on the audit log.
         overwrites: :class:`dict`
             A :class:`dict` of target (either a role or a member) to
             :class:`PermissionOverwrite` to apply to the channel.
@@ -237,15 +239,49 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
         HTTPException
             Editing the channel failed.
         """
-        await self._edit(options, reason=reason)
+        await self._edit(options)
 
     @utils.copy_doc(discord.abc.GuildChannel.clone)
-    async def clone(self, *, name=None, reason=None):
+    async def clone(self, *, name=None):
         return await self._clone_impl({
             'topic': self.topic,
             'nsfw': self.nsfw,
             'rate_limit_per_user': self.slowmode_delay
-        }, name=name, reason=reason)
+        }, name=name)
+
+    async def delete_messages(self, messages):
+        """|coro|
+
+        Deletes a list of messages. This is similar to :meth:`Message.delete`
+        except it bulk deletes multiple messages.
+
+        You must have the :attr:`~Permissions.manage_messages` permission to
+        use this (unless they're your own).
+
+        .. note::
+            Users do not have access to the message bulk-delete endpoint.
+            Since messages are just iterated over and deleted one-by-one,
+            it's easy to get ratelimited using this method.
+
+        Parameters
+        -----------
+        messages: Iterable[:class:`abc.Snowflake`]
+            An iterable of messages denoting which ones to bulk delete.
+
+        Raises
+        ------
+        Forbidden
+            You do not have proper permissions to delete the messages.
+        HTTPException
+            Deleting the messages failed.
+        """
+        if not isinstance(messages, (list, tuple)):
+            messages = list(messages)
+
+        if len(messages) == 0:
+            return # do nothing
+
+        await _delete_messages(self._state, self.id, messages)
 
     async def purge(self, *, limit=100, check=None, before=None, after=None, around=None, oldest_first=False):
         """|coro|
@@ -254,10 +290,8 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
         ``check``. If a ``check`` is not provided then all messages are deleted
         without discrimination.
 
-        You must have the :attr:`~Permissions.manage_messages` permission to
-        delete messages even if they are your own (unless you are a user
-        account). The :attr:`~Permissions.read_message_history` permission is
-        also needed to retrieve message history.
+        The :attr:`~Permissions.read_message_history` permission is needed to
+        retrieve message history.
 
         Examples
         ---------
@@ -303,47 +337,28 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
         if check is None:
             check = lambda m: True
 
+        state = self._state
+        channel_id = self.id
         iterator = self.history(limit=limit, before=before, after=after, oldest_first=oldest_first, around=around)
         ret = []
         count = 0
 
-        minimum_time = int((time.time() - 14 * 24 * 60 * 60) * 1000.0 - 1420070400000) << 22
-        strategy = _single_delete_strategy
-
         while True:
             try:
                 msg = await iterator.next()
             except NoMoreItems:
-                # no more messages to poll
-                if count >= 2:
-                    # more than 2 messages -> bulk delete
-                    to_delete = ret[-count:]
-                    await strategy(to_delete)
-                elif count == 1:
-                    # delete a single message
-                    await ret[-1].delete()
-
+                to_delete = ret[-count:]
+                await _delete_messages(state, channel_id, to_delete)
                 return ret
             else:
                 if count == 100:
                     # we've reached a full 'queue'
                     to_delete = ret[-100:]
-                    await strategy(to_delete)
+                    await _delete_messages(state, to_delete)
                     count = 0
                     await asyncio.sleep(1)
 
                 if check(msg):
-                    if msg.id < minimum_time:
-                        # older than 14 days old
-                        if count == 1:
-                            await ret[-1].delete()
-                        elif count >= 2:
-                            to_delete = ret[-count:]
-                            await strategy(to_delete)
-
-                        count = 0
-                        strategy = _single_delete_strategy
-
                     count += 1
                     ret.append(msg)
 
@@ -369,16 +384,13 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
         data = await self._state.http.channel_webhooks(self.id)
         return [Webhook.from_state(d, state=self._state) for d in data]
 
-    async def create_webhook(self, *, name, avatar=None, reason=None):
+    async def create_webhook(self, *, name, avatar=None):
         """|coro|
 
         Creates a webhook for this channel.
 
         Requires :attr:`~.Permissions.manage_webhooks` permissions.
 
-        .. versionchanged:: 1.1
-            Added the ``reason`` keyword-only parameter.
-
         Parameters
         -------------
         name: :class:`str`
@@ -386,8 +398,6 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
         avatar: Optional[:class:`bytes`]
             A :term:`py:bytes-like object` representing the webhook's default avatar.
             This operates similarly to :meth:`~ClientUser.edit`.
-        reason: Optional[:class:`str`]
-            The reason for creating this webhook. Shows up in the audit logs.
 
         Raises
         -------
@@ -406,10 +416,10 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
         if avatar is not None:
             avatar = utils._bytes_to_base64_data(avatar)
 
-        data = await self._state.http.create_webhook(self.id, name=str(name), avatar=avatar, reason=reason)
+        data = await self._state.http.create_webhook(self.id, name=str(name), avatar=avatar)
         return Webhook.from_state(data, state=self._state)
 
-    async def follow(self, *, destination, reason=None):
+    async def follow(self, *, destination):
         """
         Follows a channel using a webhook.
 
@@ -426,10 +436,6 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
         -----------
         destination: :class:`TextChannel`
             The channel you would like to follow from.
-        reason: Optional[:class:`str`]
-            The reason for following the channel. Shows up on the destination guild's audit log.
-
-            .. versionadded:: 1.4
 
         Raises
         -------
@@ -451,7 +457,7 @@ class TextChannel(discord.abc.Messageable, discord.abc.GuildChannel, Hashable):
             raise InvalidArgument('Expected TextChannel received {0.__name__}'.format(type(destination)))
 
         from .webhook import Webhook
-        data = await self._state.http.follow_webhook(self.id, webhook_channel_id=destination.id, reason=reason)
+        data = await self._state.http.follow_webhook(self.id, webhook_channel_id=destination.id)
         return Webhook._as_follower(data, channel=destination, user=self._state.user)
 
     def get_partial_message(self, message_id):
@@ -614,13 +620,13 @@ class VoiceChannel(VocalGuildChannel):
         return ChannelType.voice
 
     @utils.copy_doc(discord.abc.GuildChannel.clone)
-    async def clone(self, *, name=None, reason=None):
+    async def clone(self, *, name=None):
         return await self._clone_impl({
             'bitrate': self.bitrate,
             'user_limit': self.user_limit
-        }, name=name, reason=reason)
+        }, name=name)
 
-    async def edit(self, *, reason=None, **options):
+    async def edit(self, **options):
         """|coro|
 
         Edits the channel.
@@ -647,8 +653,6 @@ class VoiceChannel(VocalGuildChannel):
         category: Optional[:class:`CategoryChannel`]
             The new category for this channel. Can be ``None`` to remove the
             category.
-        reason: Optional[:class:`str`]
-            The reason for editing this channel. Shows up on the audit log.
         overwrites: :class:`dict`
             A :class:`dict` of target (either a role or a member) to
             :class:`PermissionOverwrite` to apply to the channel.
@@ -668,7 +672,7 @@ class VoiceChannel(VocalGuildChannel):
             Editing the channel failed.
         """
 
-        await self._edit(options, reason=reason)
+        await self._edit(options)
 
 class StageChannel(VocalGuildChannel):
     """Represents a Discord guild stage channel.
@@ -746,12 +750,12 @@ class StageChannel(VocalGuildChannel):
         return ChannelType.stage_voice
 
     @utils.copy_doc(discord.abc.GuildChannel.clone)
-    async def clone(self, *, name=None, reason=None):
+    async def clone(self, *, name=None):
         return await self._clone_impl({
             'topic': self.topic,
-        }, name=name, reason=reason)
+        }, name=name)
 
-    async def edit(self, *, reason=None, **options):
+    async def edit(self, **options):
         """|coro|
 
         Edits the channel.
@@ -773,8 +777,6 @@ class StageChannel(VocalGuildChannel):
         category: Optional[:class:`CategoryChannel`]
             The new category for this channel. Can be ``None`` to remove the
             category.
-        reason: Optional[:class:`str`]
-            The reason for editing this channel. Shows up on the audit log.
         overwrites: :class:`dict`
             A :class:`dict` of target (either a role or a member) to
             :class:`PermissionOverwrite` to apply to the channel.
@@ -792,7 +794,7 @@ class StageChannel(VocalGuildChannel):
             Editing the channel failed.
         """
 
-        await self._edit(options, reason=reason)
+        await self._edit(options)
 
 class CategoryChannel(discord.abc.GuildChannel, Hashable):
     """Represents a Discord channel category.
@@ -862,12 +864,12 @@ class CategoryChannel(discord.abc.GuildChannel, Hashable):
         return self.nsfw
 
     @utils.copy_doc(discord.abc.GuildChannel.clone)
-    async def clone(self, *, name=None, reason=None):
+    async def clone(self, *, name=None):
         return await self._clone_impl({
             'nsfw': self.nsfw
-        }, name=name, reason=reason)
+        }, name=name)
 
-    async def edit(self, *, reason=None, **options):
+    async def edit(self, **options):
         """|coro|
 
         Edits the channel.
@@ -886,8 +888,6 @@ class CategoryChannel(discord.abc.GuildChannel, Hashable):
             The new category's position.
         nsfw: :class:`bool`
             To mark the category as NSFW or not.
-        reason: Optional[:class:`str`]
-            The reason for editing this category. Shows up on the audit log.
         overwrites: :class:`dict`
             A :class:`dict` of target (either a role or a member) to
             :class:`PermissionOverwrite` to apply to the channel.
@@ -902,7 +902,7 @@ class CategoryChannel(discord.abc.GuildChannel, Hashable):
             Editing the category failed.
         """
 
-        await self._edit(options=options, reason=reason)
+        await self._edit(options=options)
 
     @utils.copy_doc(discord.abc.GuildChannel.move)
     async def move(self, **kwargs):
@@ -952,7 +952,7 @@ class CategoryChannel(discord.abc.GuildChannel, Hashable):
         ret.sort(key=lambda c: (c.position, c.id))
         return ret
 
-    async def create_text_channel(self, name, *, overwrites=None, reason=None, **options):
+    async def create_text_channel(self, name, *, overwrites=None, **options):
         """|coro|
 
         A shortcut method to :meth:`Guild.create_text_channel` to create a :class:`TextChannel` in the category.
@@ -962,9 +962,9 @@ class CategoryChannel(discord.abc.GuildChannel, Hashable):
         :class:`TextChannel`
             The channel that was just created.
         """
-        return await self.guild.create_text_channel(name, overwrites=overwrites, category=self, reason=reason, **options)
+        return await self.guild.create_text_channel(name, overwrites=overwrites, category=self, **options)
 
-    async def create_voice_channel(self, name, *, overwrites=None, reason=None, **options):
+    async def create_voice_channel(self, name, *, overwrites=None, **options):
         """|coro|
 
         A shortcut method to :meth:`Guild.create_voice_channel` to create a :class:`VoiceChannel` in the category.
@@ -974,9 +974,9 @@ class CategoryChannel(discord.abc.GuildChannel, Hashable):
         :class:`VoiceChannel`
             The channel that was just created.
         """
-        return await self.guild.create_voice_channel(name, overwrites=overwrites, category=self, reason=reason, **options)
+        return await self.guild.create_voice_channel(name, overwrites=overwrites, category=self, **options)
 
-    async def create_stage_channel(self, name, *, overwrites=None, reason=None, **options):
+    async def create_stage_channel(self, name, *, overwrites=None, **options):
         """|coro|
 
         A shortcut method to :meth:`Guild.create_stage_channel` to create a :class:`StageChannel` in the category.
@@ -988,7 +988,7 @@ class CategoryChannel(discord.abc.GuildChannel, Hashable):
         :class:`StageChannel`
             The channel that was just created.
         """
-        return await self.guild.create_stage_channel(name, overwrites=overwrites, category=self, reason=reason, **options)
+        return await self.guild.create_stage_channel(name, overwrites=overwrites, category=self, **options)
 
 class StoreChannel(discord.abc.GuildChannel, Hashable):
     """Represents a Discord guild store channel.
@@ -1067,12 +1067,12 @@ class StoreChannel(discord.abc.GuildChannel, Hashable):
         return self.nsfw
 
     @utils.copy_doc(discord.abc.GuildChannel.clone)
-    async def clone(self, *, name=None, reason=None):
+    async def clone(self, *, name=None):
         return await self._clone_impl({
             'nsfw': self.nsfw
-        }, name=name, reason=reason)
+        }, name=name)
 
-    async def edit(self, *, reason=None, **options):
+    async def edit(self, **options):
         """|coro|
 
         Edits the channel.
@@ -1094,8 +1094,6 @@ class StoreChannel(discord.abc.GuildChannel, Hashable):
         category: Optional[:class:`CategoryChannel`]
             The new category for this channel. Can be ``None`` to remove the
             category.
-        reason: Optional[:class:`str`]
-            The reason for editing this channel. Shows up on the audit log.
         overwrites: :class:`dict`
             A :class:`dict` of target (either a role or a member) to
             :class:`PermissionOverwrite` to apply to the channel.
@@ -1112,7 +1110,7 @@ class StoreChannel(discord.abc.GuildChannel, Hashable):
         HTTPException
             Editing the channel failed.
         """
-        await self._edit(options, reason=reason)
+        await self._edit(options)
 
 class DMChannel(discord.abc.Messageable, Hashable):
     """Represents a Discord direct message channel.
@@ -1149,10 +1147,7 @@ class DMChannel(discord.abc.Messageable, Hashable):
 
     def __init__(self, *, me, state, data):
         self._state = state
-        if 'recipients' in data:
-            self.recipient = state.store_user(data['recipients'][0])
-        else:
-            self.recipient = state.store_lazy_user(data['recipient_ids'][0])
+        self.recipient = state.store_user(data['recipients'][0])
         self.me = me
         self.id = int(data['id'])
 
@@ -1200,6 +1195,7 @@ class DMChannel(discord.abc.Messageable, Hashable):
         """
 
         base = Permissions.text()
+        base.read_messages = True
         base.send_tts_messages = False
         base.manage_messages = False
         return base
@@ -1277,10 +1273,7 @@ class GroupChannel(discord.abc.Messageable, Hashable):
         self.name = data.get('name')
 
         try:
-            if 'recipients' in data:
-                self.recipients = [self._state.store_user(u) for u in data['recipients']]
-            else:
-                self.recipients = [self._state.store_lazy_user(u) for u in data['recipient_ids']]
+            self.recipients = [self._state.store_user(u) for u in data['recipients']]
         except KeyError:
             pass
 
@@ -1376,6 +1369,7 @@ class GroupChannel(discord.abc.Messageable, Hashable):
         """
 
         base = Permissions.text()
+        base.read_messages = True
         base.send_tts_messages = False
         base.manage_messages = False
         base.mention_everyone = True
@@ -1385,7 +1379,6 @@ class GroupChannel(discord.abc.Messageable, Hashable):
 
         return base
 
-    @utils.deprecated()
     async def add_recipients(self, *recipients):
         r"""|coro|
 
@@ -1396,8 +1389,6 @@ class GroupChannel(discord.abc.Messageable, Hashable):
         add a recipient to the group, you must have a relationship
         with the user of type :attr:`RelationshipType.friend`.
 
-        .. deprecated:: 1.7
-
         Parameters
         -----------
         \*recipients: :class:`User`
@@ -1415,14 +1406,11 @@ class GroupChannel(discord.abc.Messageable, Hashable):
         for recipient in recipients:
             await req(self.id, recipient.id)
 
-    @utils.deprecated()
     async def remove_recipients(self, *recipients):
         r"""|coro|
 
         Removes recipients from this group.
 
-        .. deprecated:: 1.7
-
         Parameters
         -----------
         \*recipients: :class:`User`
@@ -1440,14 +1428,11 @@ class GroupChannel(discord.abc.Messageable, Hashable):
         for recipient in recipients:
             await req(self.id, recipient.id)
 
-    @utils.deprecated()
     async def edit(self, **fields):
         """|coro|
 
         Edits the group.
 
-        .. deprecated:: 1.7
-
         Parameters
         -----------
         name: Optional[:class:`str`]
