@@ -32,16 +32,16 @@ import traceback
 
 import aiohttp
 
-from .user import User, Profile
+from .user import User, Profile, Note
 from .invite import Invite
 from .template import Template
 from .widget import Widget
 from .guild import Guild
 from .channel import _channel_factory
-from .enums import ChannelType
+from .enums import ChannelType, Status, try_enum
 from .mentions import AllowedMentions
 from .errors import *
-from .enums import Status, VoiceRegion
+from .enums import Status
 from .gateway import *
 from .activity import BaseActivity, create_activity
 from .voice_client import VoiceClient
@@ -134,31 +134,24 @@ class Client:
         Proxy URL.
     proxy_auth: Optional[:class:`aiohttp.BasicAuth`]
         An object that represents proxy HTTP Basic Authorization.
-    shard_id: Optional[:class:`int`]
-        Integer starting at ``0`` and less than :attr:`.shard_count`.
-    shard_count: Optional[:class:`int`]
-        The total number of shards.
-    intents: :class:`Intents`
-        The intents that you want to enable for the session. This is a way of
-        disabling and enabling certain gateway events from triggering and being sent.
-        If not given, defaults to a regularly constructed :class:`Intents` class.
 
         .. versionadded:: 1.5
     member_cache_flags: :class:`MemberCacheFlags`
         Allows for finer control over how the library caches members.
-        If not given, defaults to cache as much as possible with the
-        currently selected intents.
+        If not given, defaults to cache as much as possible.
 
         .. versionadded:: 1.5
-    fetch_offline_members: :class:`bool`
-        A deprecated alias of ``chunk_guilds_at_startup``.
     chunk_guilds_at_startup: :class:`bool`
         Indicates if :func:`.on_ready` should be delayed to chunk all guilds
         at start-up if necessary. This operation is incredibly slow for large
-        amounts of guilds. The default is ``True`` if :attr:`Intents.members`
-        is ``True``.
+        amounts of guilds. The default is ``True``.
 
         .. versionadded:: 1.5
+    guild_subscription_options: :class:`GuildSubscriptionOptions`
+        Allows for control over the library's auto-subscribing.
+        If not given, defaults to "sane values".
+
+        .. versionadded:: 1.9
     status: Optional[:class:`.Status`]
         A status to start your presence with upon logging on to Discord.
     activity: Optional[:class:`.BaseActivity`]
@@ -172,42 +165,6 @@ class Client:
         WebSocket in the case of not receiving a HEARTBEAT_ACK. Useful if
         processing the initial packets take too long to the point of disconnecting
         you. The default timeout is 60 seconds.
-    guild_ready_timeout: :class:`float`
-        The maximum number of seconds to wait for the GUILD_CREATE stream to end before
-        preparing the member cache and firing READY. The default timeout is 2 seconds.
-
-        .. versionadded:: 1.4
-    guild_subscriptions: :class:`bool`
-        Whether to dispatch presence or typing events. Defaults to ``True``.
-
-        .. versionadded:: 1.3
-
-        .. warning::
-
-            If this is set to ``False`` then the following features will be disabled:
-
-                - No user related updates (:func:`on_user_update` will not dispatch)
-                - All member related events will be disabled.
-                    - :func:`on_member_update`
-                    - :func:`on_member_join`
-                    - :func:`on_member_remove`
-
-                - Typing events will be disabled (:func:`on_typing`).
-                - If ``fetch_offline_members`` is set to ``False`` then the user cache will not exist.
-                  This makes it difficult or impossible to do many things, for example:
-
-                    - Computing permissions
-                    - Querying members in a voice channel via :attr:`VoiceChannel.members` will be empty.
-                    - Most forms of receiving :class:`Member` will be
-                      receiving :class:`User` instead, except for message events.
-                    - :attr:`Guild.owner` will usually resolve to ``None``.
-                    - :meth:`Guild.get_member` will usually be unavailable.
-                    - Anything that involves using :class:`Member`.
-                    - :attr:`users` will not be as populated.
-                    - etc.
-
-            In short, this makes it so the only member you can reliably query is the
-            message author. Useful for bots that do not require any state.
     assume_unsync_clock: :class:`bool`
         Whether to assume the system clock is unsynced. This applies to the ratelimit handling
         code. If this is set to ``True``, the default, then the library uses the time to reset
@@ -228,8 +185,6 @@ class Client:
         self.ws = None
         self.loop = asyncio.get_event_loop() if loop is None else loop
         self._listeners = {}
-        self.shard_id = options.get('shard_id')
-        self.shard_count = options.get('shard_count')
 
         connector = options.pop('connector', None)
         proxy = options.pop('proxy', None)
@@ -238,7 +193,8 @@ class Client:
         self.http = HTTPClient(connector, proxy=proxy, proxy_auth=proxy_auth, unsync_clock=unsync_clock, loop=self.loop)
 
         self._handlers = {
-            'ready': self._handle_ready
+            'ready': self._handle_ready,
+            'connect': self._handle_connect
         }
 
         self._hooks = {
@@ -246,7 +202,6 @@ class Client:
         }
 
         self._connection = self._get_state(**options)
-        self._connection.shard_count = self.shard_count
         self._closed = False
         self._ready = asyncio.Event()
         self._connection._get_websocket = self._get_websocket
@@ -258,59 +213,22 @@ class Client:
 
     # internals
 
-    def _get_websocket(self, guild_id=None, *, shard_id=None):
+    def _get_websocket(self):
         return self.ws
 
     def _get_state(self, **options):
         return ConnectionState(dispatch=self.dispatch, handlers=self._handlers,
-                               hooks=self._hooks, syncer=self._syncer, http=self.http, loop=self.loop, **options)
-
-    async def _syncer(self, guilds):
-        await self.ws.request_sync(guilds)
+                               hooks=self._hooks, http=self.http, loop=self.loop, **options)
 
     def _handle_ready(self):
-        for index in range(len(self.guilds)):
-            guild = self.guilds[index]
-            if index==0:
-                payload = {
-                    "op": 14,
-                    "d": {
-                        "guild_id": str(guild.id),
-                        "typing": True,
-                        "threads": False,
-                        "activities": True,
-                        "members": [],
-                        "channels": {
-                            str(guild.channels[0].id): [
-                                [
-                                    0,
-                                    99
-                                ]
-                            ]
-                        }
-                    }
-                }
-            else:
-                payload = {
-                    "op": 14,
-                    "d": {
-                        "guild_id": str(guild.id),
-                        "typing": True,
-                        "activities": True,
-                        "channels": {
-                            str(guild.channels[0].id): [
-                                [
-                                    0,
-                                    99
-                                ]
-                            ]
-                        }
-                    }
-                }
-            asyncio.ensure_future(self.ws.send_as_json(payload), loop=self.loop)
-
         self._ready.set()
 
+    def _handle_connect(self):
+        state = self._connection
+        activity = create_activity(state._activity)
+        status = try_enum(Status, state._status)
+        self.loop.create_task(self.change_presence(activity=activity, status=status))
+
     @property
     def latency(self):
         """:class:`float`: Measures latency between a HEARTBEAT and a HEARTBEAT_ACK in seconds.
@@ -446,69 +364,32 @@ class Client:
         print('Ignoring exception in {}'.format(event_method), file=sys.stderr)
         traceback.print_exc()
 
-    @utils.deprecated('Guild.chunk')
-    async def request_offline_members(self, *guilds):
-        r"""|coro|
-
-        Requests previously offline members from the guild to be filled up
-        into the :attr:`.Guild.members` cache. This function is usually not
-        called. It should only be used if you have the ``fetch_offline_members``
-        parameter set to ``False``.
-
-        When the client logs on and connects to the websocket, Discord does
-        not provide the library with offline members if the number of members
-        in the guild is larger than 250. You can check if a guild is large
-        if :attr:`.Guild.large` is ``True``.
-
-        .. warning::
-
-            This method is deprecated. Use :meth:`Guild.chunk` instead.
-
-        Parameters
-        -----------
-        \*guilds: :class:`.Guild`
-            An argument list of guilds to request offline members for.
-
-        Raises
-        -------
-        :exc:`.InvalidArgument`
-            If any guild is unavailable in the collection.
-        """
-        if any(g.unavailable for g in guilds):
-            raise InvalidArgument('An unavailable guild was passed.')
-
-        for guild in guilds:
-            await self._connection.chunk_guild(guild)
-
     # hooks
 
-    async def _call_before_identify_hook(self, shard_id, *, initial=False):
+    async def _call_before_identify_hook(self, *, initial=False):
         # This hook is an internal hook that actually calls the public one.
         # It allows the library to have its own hook without stepping on the
         # toes of those who need to override their own hook.
-        await self.before_identify_hook(shard_id, initial=initial)
+        await self.before_identify_hook(initial=initial)
 
-    async def before_identify_hook(self, shard_id, *, initial=False):
+    async def before_identify_hook(self, *, initial=False):
         """|coro|
 
         A hook that is called before IDENTIFYing a session. This is useful
         if you wish to have more control over the synchronization of multiple
         IDENTIFYing clients.
 
-        The default implementation sleeps for 5 seconds.
+        The default implementation does nothing.
 
         .. versionadded:: 1.4
 
         Parameters
         ------------
-        shard_id: :class:`int`
-            The shard ID that requested being IDENTIFY'd
         initial: :class:`bool`
             Whether this IDENTIFY is the first initial IDENTIFY.
         """
 
-        if not initial:
-            await asyncio.sleep(5.0)
+        pass
 
     # login state management
 
@@ -527,8 +408,7 @@ class Client:
         Parameters
         -----------
         token: :class:`str`
-            The authentication token. Do not prefix this token with
-            anything as the library will do it for you.
+            The authentication token.
 
         Raises
         ------
@@ -540,16 +420,15 @@ class Client:
             passing status code.
         """
 
-        log.info('logging in using static token')
+        log.info('Logging in using static token')
         await self.http.static_login(token.strip())
-        self._connection.is_bot = False
 
     @utils.deprecated('Client.close')
     async def logout(self):
         """|coro|
 
         Logs out of Discord and closes all connections.
-        
+
         .. deprecated:: 1.7
 
         .. note::
@@ -574,7 +453,7 @@ class Client:
             If we should attempt reconnecting, either due to internet
             failure or a specific failure on Discord's part. Certain
             disconnects that lead to bad state will not be handled (such as
-            invalid sharding payloads or bad tokens).
+            bad tokens).
 
         Raises
         -------
@@ -588,7 +467,6 @@ class Client:
         backoff = ExponentialBackoff()
         ws_params = {
             'initial': True,
-            'shard_id': self.shard_id,
         }
         while not self.is_closed():
             try:
@@ -626,12 +504,10 @@ class Client:
                     continue
 
                 # We should only get this when an unhandled close code happens,
-                # such as a clean disconnect (1000) or a bad state (bad token, no sharding, etc)
+                # such as a clean disconnect (1000) or a bad state (bad token, etc)
                 # sometimes, discord sends us 1000 for unknown reasons so we should reconnect
                 # regardless and rely on is_closed instead
                 if isinstance(exc, ConnectionClosed):
-                    if exc.code == 4014:
-                        raise PrivilegedIntentsRequired(exc.shard_id) from None
                     if exc.code != 1000:
                         await self.close()
                         raise
@@ -794,14 +670,6 @@ class Client:
         else:
             raise TypeError('allowed_mentions must be AllowedMentions not {0.__class__!r}'.format(value))
 
-    @property
-    def intents(self):
-        """:class:`~discord.Intents`: The intents configured for this connection.
-
-        .. versionadded:: 1.5
-        """
-        return self._connection.intents
-
     # helpers/getters
 
     @property
@@ -1071,7 +939,7 @@ class Client:
             Indicates what status to change to. If ``None``, then
             :attr:`.Status.online` is used.
         afk: Optional[:class:`bool`]
-            Indicates if you are going AFK. This allows the discord
+            Indicates if you are going AFK. This allows the Discord
             client to know how to handle push notifications better
             for you in case you are actually idle and not lying.
 
@@ -1093,6 +961,12 @@ class Client:
 
         await self.ws.change_presence(activity=activity, status=status, afk=afk)
 
+        if status:
+            try:
+                await self._connection.user.edit_settings(status=status_enum)
+            except:
+                pass
+
         for guild in self._connection.guilds:
             me = guild.me
             if me is None:
@@ -1107,7 +981,7 @@ class Client:
 
     # Guild stuff
 
-    def fetch_guilds(self, *, limit=100, before=None, after=None):
+    def fetch_guilds(self):
         """Retrieves an :class:`.AsyncIterator` that enables receiving your guilds.
 
         .. note::
@@ -1124,30 +998,14 @@ class Client:
 
         Usage ::
 
-            async for guild in client.fetch_guilds(limit=150):
+            async for guild in client.fetch_guilds():
                 print(guild.name)
 
         Flattening into a list ::
 
-            guilds = await client.fetch_guilds(limit=150).flatten()
+            guilds = await client.fetch_guilds().flatten()
             # guilds is now a list of Guild...
 
-        All parameters are optional.
-
-        Parameters
-        -----------
-        limit: Optional[:class:`int`]
-            The number of guilds to retrieve.
-            If ``None``, it retrieves every guild you have access to. Note, however,
-            that this would make it a slow operation.
-            Defaults to ``100``.
-        before: Union[:class:`.abc.Snowflake`, :class:`datetime.datetime`]
-            Retrieves guilds before this date or object.
-            If a date is provided it must be a timezone-naive datetime representing UTC time.
-        after: Union[:class:`.abc.Snowflake`, :class:`datetime.datetime`]
-            Retrieve guilds after this date or object.
-            If a date is provided it must be a timezone-naive datetime representing UTC time.
-
         Raises
         ------
         :exc:`.HTTPException`
@@ -1158,7 +1016,7 @@ class Client:
         :class:`.Guild`
             The guild with the guild data parsed.
         """
-        return GuildIterator(self, limit=limit, before=before, after=after)
+        return GuildIterator(self)
 
     async def fetch_template(self, code):
         """|coro|
@@ -1193,8 +1051,7 @@ class Client:
 
         .. note::
 
-            Using this, you will **not** receive :attr:`.Guild.channels`, :attr:`.Guild.members`,
-            :attr:`.Member.activity` and :attr:`.Member.voice` per :class:`.Member`.
+            Using this, you will **not** receive :attr:`.Guild.channels` and :attr:`.Guild.members`.
 
         .. note::
 
@@ -1220,7 +1077,7 @@ class Client:
         data = await self.http.get_guild(guild_id)
         return Guild(data=data, state=self._connection)
 
-    async def create_guild(self, name, region=None, icon=None, *, code=None):
+    async def create_guild(self, name, icon=None, *, code=None):
         """|coro|
 
         Creates a :class:`.Guild`.
@@ -1229,9 +1086,6 @@ class Client:
         ----------
         name: :class:`str`
             The name of the guild.
-        region: :class:`.VoiceRegion`
-            The region for the voice communication server.
-            Defaults to :attr:`.VoiceRegion.us_west`.
         icon: :class:`bytes`
             The :term:`py:bytes-like object` representing the icon. See :meth:`.ClientUser.edit`
             for more details on what is expected.
@@ -1256,18 +1110,76 @@ class Client:
         if icon is not None:
             icon = utils._bytes_to_base64_data(icon)
 
-        region = region or VoiceRegion.us_west
-        region_value = region.value
-
         if code:
-            data = await self.http.create_from_template(code, name, region_value, icon)
+            data = await self.http.create_from_template(code, name, icon)
         else:
-            data = await self.http.create_guild(name, region_value, icon)
+            data = await self.http.create_guild(name, icon)
         return Guild(data=data, state=self._connection)
 
+    async def join_guild(self, invite):
+        """|coro|
+
+        Joines a guild.
+
+        .. versionadded:: 1.9
+
+        Parameters
+        ----------
+        invite: Union[:class:`.Invite`, :class:`str`]
+            The Discord invite ID, URL (must be a discord.gg URL), or :class:`.Invite`.
+
+        Raises
+        ------
+        :exc:`.HTTPException`
+            Joining the guild failed.
+        :exc:`.InvalidArgument`
+            Tried to join a guild you're already in.
+
+        Returns
+        -------
+        :class:`.Guild`
+            The guild joined. This is not the same guild that is
+            added to cache.
+        """
+
+        if not isinstance(invite, Invite):
+            invite = await self.fetch_invite(invite, with_counts=False, with_expiration=False)
+
+        data = await self.http.join_guild(invite.code, guild_id=invite.guild.id, channel_id=invite.channel.id, channel_type=invite.channel.type.value)
+
+        new_member = data.get('new_member', False)
+        if not new_member:
+            raise InvalidArgument('Tried to join a guild you\'re already in.')
+
+        return Guild(data=data['guild'], state=self._connection)
+
+    async def leave_guild(self, guild_id):
+        """|coro|
+
+        Leaves a guild.
+
+        .. note::
+
+            You cannot leave a guild that you own, you must delete it instead.
+
+        .. versionadded:: 1.9
+
+        Parameters
+        ----------
+        guild_id:
+            The ID of the guild you want to leave.
+
+        Raises
+        ------
+        :exc:`.HTTPException`
+            Leaving the guild failed.
+        """
+
+        await self.http.leave_guild(guild_id)
+
     # Invite management
 
-    async def fetch_invite(self, url, *, with_counts=True):
+    async def fetch_invite(self, url, *, with_counts=True, with_expiration=True):
         """|coro|
 
         Gets an :class:`.Invite` from a discord.gg URL or ID.
@@ -1286,6 +1198,11 @@ class Client:
             Whether to include count information in the invite. This fills the
             :attr:`.Invite.approximate_member_count` and :attr:`.Invite.approximate_presence_count`
             fields.
+        with_expiration: :class:`bool`
+            Whether to include expiration information in the invite. This fills the
+            :attr:`.Invite.expires_at` fields.
+
+            ..versionadded:: 1.9
 
         Raises
         -------
@@ -1301,7 +1218,7 @@ class Client:
         """
 
         invite_id = utils.resolve_invite(url)
-        data = await self.http.get_invite(invite_id, with_counts=with_counts)
+        data = await self.http.get_invite(invite_id, with_counts=with_counts, with_expiration=with_expiration)
         return Invite.from_incomplete(state=self._connection, data=data)
 
     async def delete_invite(self, invite):
@@ -1332,6 +1249,59 @@ class Client:
 
     # Miscellaneous stuff
 
+    async def fetch_application(self, app_id):
+        """|coro|
+
+        Retrieves a :class:`AppInfo` for a specified ID.
+
+        .. versionadded:: 1.9
+
+        Parameters
+        ----------
+        app_id: :class:`int`
+            The ID of the application to retrieve.
+
+        Raises
+        -------
+        :exc:`.HTTPException`
+            Fetching applications failed.
+
+        Returns
+        --------
+        :class:`AppInfo`
+            The application.
+        """
+
+        data = await self.http.get_applications(app_id)
+        return AppInfo(self._connection, data)
+
+    async def fetch_applications(self, *, with_team_applications=True):
+        """|coro|
+
+        Gets a List[:class:`AppInfo`] of all your applications.
+
+        .. versionadded:: 1.9
+
+        Parameters
+        ----------
+        with_team_applications: :class:`bool`
+            Whether to include team applications.
+
+        Raises
+        -------
+        :exc:`.HTTPException`
+            Fetching applications failed.
+
+        Returns
+        --------
+        List[:class:`AppInfo`]
+            All your applications.
+        """
+
+        data = await self.http.get_applications(with_team_applications=with_team_applications)
+
+        return [AppInfo(self._connection, app) for app in data]
+
     async def fetch_widget(self, guild_id):
         """|coro|
 
@@ -1359,43 +1329,63 @@ class Client:
             The guild's widget.
         """
         data = await self.http.get_widget(guild_id)
-
         return Widget(state=self._connection, data=data)
 
-    async def application_info(self):
+    async def fetch_user(self, user_id):
         """|coro|
 
-        Retrieves the bot's application information.
+        Retrieves a :class:`~discord.User` based on their ID.
+        You do not have to share any guilds with the user to get
+        this information, however many operations require that you do.
+
+        .. note::
+
+            This method is an API call. If you have member cache enabled, consider :meth:`get_user` instead.
+
+        Parameters
+        -----------
+        user_id: :class:`int`
+            The user's ID to fetch from.
 
         Raises
         -------
+        :exc:`.NotFound`
+            A user with this ID does not exist.
         :exc:`.HTTPException`
-            Retrieving the information failed somehow.
+            Fetching the user failed.
 
         Returns
         --------
-        :class:`.AppInfo`
-            The bot's application information.
+        :class:`~discord.User`
+            The user you requested.
         """
-        data = await self.http.application_info()
-        if 'rpc_origins' not in data:
-            data['rpc_origins'] = None
-        return AppInfo(self._connection, data)
+        data = await self.http.get_user(user_id)
+        return User(state=self._connection, data=data)
 
-    async def fetch_user_profile(self, user_id):
+    async def fetch_user_profile(self, user_id, *, with_mutuals=True, fetch_note=True):
         """|coro|
 
         Gets an arbitrary user's profile.
 
+        You must share a guild or be friends with this user to
+        get this information.
+
         Parameters
         ------------
         user_id: :class:`int`
             The ID of the user to fetch their profile for.
+        with_mutuals: :class:`bool`
+            Whether to fetch mutual guilds and friends.
+            This fills in :attr:`mutual_guilds` & :attr:`mutual_friends`.
+        fetch_note: :class:`bool`
+            Whether to pre-fetch the user's note.
 
         Raises
         -------
+        :exc:`.NotFound`
+            A user with this ID does not exist.
         :exc:`.Forbidden`
-            Not allowed to fetch profiles.
+            Not allowed to fetch this profile.
         :exc:`.HTTPException`
             Fetching the profile failed.
 
@@ -1406,19 +1396,19 @@ class Client:
         """
 
         state = self._connection
-        data = await self.http.get_user_profile(user_id)
+        data = await self.http.get_user_profile(user_id, with_mutual_guilds=with_mutuals)
 
-        def transform(d):
-            return state._get_guild(int(d['id']))
+        if with_mutuals:
+            data['mutual_friends'] = await self.http.get_mutual_friends(user_id)
 
-        since = data.get('premium_since')
-        mutual_guilds = list(filter(None, map(transform, data.get('mutual_guilds', []))))
-        user = data['user']
-        return Profile(flags=user.get('flags', 0),
-                       premium_since=utils.parse_time(since),
-                       mutual_guilds=mutual_guilds,
-                       user=User(data=user, state=state),
-                       connected_accounts=data['connected_accounts'])
+        profile = Profile(state, data)
+
+        if fetch_note:
+            await profile.note.fetch()
+
+        return profile
+
+    fetch_profile = fetch_user_profile
 
     async def fetch_channel(self, channel_id):
         """|coro|
@@ -1468,7 +1458,7 @@ class Client:
         Retrieves a :class:`.Webhook` with the specified ID.
 
         Raises
-        --------
+        -------
         :exc:`.HTTPException`
             Retrieving the webhook failed.
         :exc:`.NotFound`
@@ -1483,3 +1473,33 @@ class Client:
         """
         data = await self.http.get_webhook(webhook_id)
         return Webhook.from_state(data, state=self._connection)
+
+    async def fetch_notes(self):
+        """|coro|
+
+        Retrieves a list of :class:`Note` objects representing all your notes.
+
+        Raises
+        -------
+        :exc:`.HTTPException`
+            Retreiving the notes failed.
+        """
+        state = self._connection
+        data = await self.http.get_notes()
+        return [Note(state, int(id), note=note) for id, note in data.items()]
+
+    async def fetch_note(self, user_id):
+        """|coro|
+
+        Retrieves a :class:`Note` for the specified user ID.
+
+        Raises
+        -------
+        :exc:`.HTTPException`
+            Retreiving the note failed.
+        """
+        try:
+            data = await self.http.get_note(user_id)
+        except NotFound:
+            data = {'note': ''}
+        return Note(self._connection, int(user_id), note=data['note'])
