@@ -24,19 +24,270 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.
 """
 
-from collections import namedtuple
-
 import discord.abc
 from .flags import PublicUserFlags
-from .utils import snowflake_time, _bytes_to_base64_data, parse_time
-from .enums import DefaultAvatar, RelationshipType, UserFlags, HypeSquadHouse, PremiumType, try_enum
-from .errors import ClientException
+from .utils import snowflake_time, _bytes_to_base64_data, parse_time, cached_slot_property
+from .enums import DefaultAvatar, FriendFlags, StickerAnimationOptions, Theme, UserContentFilter, RelationshipAction, RelationshipType, UserFlags, HypeSquadHouse, PremiumType, try_enum
+from .errors import ClientException, NotFound
 from .colour import Colour
 from .asset import Asset
-from .utils import deprecated
+from .settings import Settings
+from .object import Object
 
-class Profile(namedtuple('Profile', 'flags user mutual_guilds connected_accounts premium_since')):
-    __slots__ = ()
+class Note:
+    """Represents a Discord note."""
+    __slots__ = ('_state', '_note', '_user_id', '_user')
+
+    def __init__(self, state, user_id, *, user=None, note=0):
+        self._state = state
+        self._user_id = user_id
+        self._note = note
+        if user:
+            self._user = user
+
+    @property
+    def note(self):
+        """Returns the note.
+
+        Raises
+        -------
+        ClientException
+            Attempted to access note without fetching it.
+        """
+        if note == 0:
+            raise ClientException('Note is not fetched.')
+        return self._note
+
+    @cached_slot_property('_user')
+    def user(self):
+        """Returns the :class:`User` the note belongs to.
+
+        If the user isn't in the cache, it returns a
+        :class:`Object` instead.
+        """
+        state = self._state
+        user_id = self._user_id
+
+        user = state.get_user(user_id)
+        if user is None:
+            user = Object(user_id)
+        return user
+
+    async def fetch(self):
+        """|coro|
+
+        Retrieves the note.
+
+        Raises
+        -------
+        HTTPException
+            Fetching the note failed.
+
+        Returns
+        --------
+        :class:`str`
+            The note.
+        """
+        try:
+            data = await self._state.http.get_note(self.user.id)
+            self._note = data['note']
+            return data['note']
+        except NotFound: # 404 = no note
+            self._note = None
+            return None
+
+    async def edit(self, note):
+        """|coro|
+
+        Changes the note.
+
+        Raises
+        -------
+        HTTPException
+            Changing the note failed.
+        """
+        await self._state.http.set_note(self._user_id, note=note)
+        self._note = note
+
+    async def delete(self):
+        """|coro|
+
+        A shortcut to :meth:`.edit` that deletes the note.
+
+        Raises
+        -------
+        HTTPException
+            Deleting the note failed.
+        """
+        await self.edit(None)
+        self._note = None
+
+    def __str__(self):
+        note = self._note
+        if note == 0:
+            raise ClientException('Note is not fetched.')
+        elif note is None:
+            return ''
+        else:
+            return note
+
+    def __repr__(self):
+        base = f'<Note user={self.user!r}'
+        note = self._note
+        if note != 0:
+            base += f' note={note or ""}>'
+        else:
+            base += '>'
+        return base
+
+    def __len__(self):
+        try:
+            return len(self._note)
+        except TypeError:
+            return 0
+
+    def __eq__(self, other):
+        try:
+            return isinstance(other, Note) and self.note == other.note
+        except TypeError:
+            return False
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __bool__(self):
+        try:
+            return bool(self._note)
+        except TypeError:
+            return False
+
+class Profile:
+    """Represents a Discord profile.
+
+    Attributes
+    ----------
+    flags: :class:`int`
+        The user's flags. Will be its own class (like public_flags) in the future.
+    bio: Optional[:class:`str`]
+        The user's "about me" field. Could be ``None``.
+    banner: Optional[:class:`str`]
+        The banner hash the user has. Could be ``None``.
+    user: :class:`User`
+        The user the profile represents.
+    premium_since: Optional[:class:`datetime.datetime`]
+        A datetime object denoting how long a user has been premium (had Nitro).
+        Could be ``None``.
+    connected_accounts: Optional[List[:class:`dict`]]
+        The connected accounts that show up on the profile.
+        These are currently just the raw json, but this will change in the future.
+    note: :class:`Note`
+        Represents the note on the profile.
+    """
+    def __init__(self, state, data):
+        self._state = state
+
+        user = data['user']
+        self.flags = user.pop('flags', 0) # TODO: Figure out the differences and parse them
+        bio = user.pop('bio')
+        self.bio = bio if bio else None
+        self.banner = user.pop('banner')
+        self._accent_color = user.pop('accent_color')
+        self.user = user = User(data=user, state=state)
+
+        self.premium_since = parse_time(data['premium_since'])
+        self.connected_accounts = data['connected_accounts']
+
+        self.note = Note(state, user.id, user=user)
+
+        if 'mutual_guilds' in data:
+            self.mutual_guilds = self._parse_mutual_guilds(data['mutual_guilds'])
+        if 'mutual_friends' in data:
+            self.mutual_friends = self._parse_mutual_friends(data['mutual_friends'])
+
+    def __str__(self):
+        return '{0.name}#{0.discriminator}'.format(self.user)
+
+    def __repr__(self):
+        return '<Profile user={0.user!r} bio={0.bio}>'.format(self)
+
+    def _parse_mutual_guilds(self, mutual_guilds):
+        state = self._state
+
+        def get_guild(guild):
+            return state._get_guild(int(guild['id']))
+
+        return list(filter(None, map(get_guild, mutual_guilds)))
+
+    def _parse_mutual_friends(self, mutual_friends):
+        state = self._state
+        return [state.store_user(friend) for friend in mutual_friends]
+
+    @property
+    def banner_url(self):
+        """:class:`Asset`: Returns an :class:`Asset` for the banner the user has.
+
+        This is equivalent to calling :meth:`banner_url_as` with
+        the default parameters (i.e. webp/gif detection and a size of 1024).
+
+        .. versionadded:: 1.9
+        """
+        return self.banner_url_as(format=None, size=1024)
+
+    def is_banner_animated(self):
+        """:class:`bool`: Indicates if the user has an animated banner.
+
+        .. versionadded:: 1.9
+        """
+        return bool(self.banner and self.banner.startswith('a_'))
+
+    def banner_url_as(self, *, format=None, static_format='webp', size=1024):
+        """Returns an :class:`Asset` for the banner the user has.
+
+        The format must be one of 'webp', 'jpeg', 'jpg', 'png' or 'gif', and
+        'gif' is only valid for animated banner. The size must be a power of
+        2 between 16 and 4096.
+
+        Parameters
+        -----------
+        format: Optional[:class:`str`]
+            The format to attempt to convert the avatar to.
+            If the format is ``None``, then it is automatically
+            detected into either 'gif' or static_format depending on the
+            banner being animated or not.
+        static_format: Optional[:class:`str`]
+            Format to attempt to convert only non-animated banners to.
+            Defaults to 'webp'
+        size: :class:`int`
+            The size of the image to display.
+
+        Raises
+        ------
+        InvalidArgument
+            Bad image format passed to ``format`` or ``static_format``, or
+            invalid ``size``.
+
+        Returns
+        --------
+        :class:`Asset`
+            The resulting CDN asset.
+
+        .. versionadded:: 1.9
+        """
+        return Asset._from_user_banner(self.user._state, self, format=format, static_format=static_format, size=size)
+
+    @property
+    def accent_colour(self):
+        """:class:`Colour`: Returns the user's accent colour.
+
+        There is an alias for this named :attr:`accent_color`.
+
+        .. versionadded:: 1.9
+        """
+        if self._accent_color is None:
+            return None
+        return Colour(self._accent_color)
+
+    accent_color = accent_colour
 
     @property
     def nitro(self):
@@ -69,9 +320,10 @@ class Profile(namedtuple('Profile', 'flags user mutual_guilds connected_accounts
         return self._has_flag(UserFlags.hypesquad)
 
     @property
-    def hypesquad_houses(self):
+    def hypesquad_house(self):
         flags = (UserFlags.hypesquad_bravery, UserFlags.hypesquad_brilliance, UserFlags.hypesquad_balance)
-        return [house for house, flag in zip(HypeSquadHouse, flags) if self._has_flag(flag)]
+        # I have no idea why this was originally a list but I'm just going to return the first item just in case.
+        return [house for house, flag in zip(HypeSquadHouse, flags) if self._has_flag(flag)][0]
 
     @property
     def team_user(self):
@@ -132,6 +384,8 @@ class BaseUser(_BaseUser):
             'avatar': self.avatar,
             'discriminator': self.discriminator,
             'bot': self.bot,
+            'system': self.system,
+            'public_flags': self._public_flags
         }
 
     @property
@@ -304,9 +558,13 @@ class ClientUser(BaseUser):
     id: :class:`int`
         The user's unique ID.
     discriminator: :class:`str`
-        The user's discriminator. This is given when the username has conflicts.
+        The user's discriminator.
+    bio: Optional[:class:`str`]
+        The user's "about me" field. Could be ``None``.
     avatar: Optional[:class:`str`]
         The avatar hash the user has. Could be ``None``.
+    banner: Optional[:class:`str`]
+        The banner hash the user has. Could be ``None``.
     bot: :class:`bool`
         Specifies if the user is a bot account.
     system: :class:`bool`
@@ -317,29 +575,31 @@ class ClientUser(BaseUser):
     verified: :class:`bool`
         Specifies if the user's email is verified.
     email: Optional[:class:`str`]
-        The email the user used when registering.
+        The email of the user.
+    phone: Optional[:class:`int`]
+        The phone number of the user.
 
     locale: Optional[:class:`str`]
         The IETF language tag used to identify the language the user is using.
     mfa_enabled: :class:`bool`
         Specifies if the user has MFA turned on and working.
     premium: :class:`bool`
-        Specifies if the user is a premium user (e.g. has Discord Nitro).
-
-        .. deprecated:: 1.7
-
+        Specifies if the user is a premium user (i.e. has Discord Nitro).
     premium_type: Optional[:class:`PremiumType`]
-        Specifies the type of premium a user has (e.g. Nitro or Nitro Classic). Could be None if the user is not premium.
-
-        .. deprecated:: 1.7
+        Specifies the type of premium a user has (i.e. Nitro or Nitro Classic). Could be None if the user is not premium.
+    settings: :class:`Settings`
+        The user's client settings.
+    note: :class:`Note`
+        The user's note. Not pre-fetched.
     """
     __slots__ = BaseUser.__slots__ + \
-                ('email', 'locale', '_flags', 'verified', 'mfa_enabled',
-                 'premium', 'premium_type', '_relationships', '__weakref__')
+                ('settings', 'bio', 'banner', '_accent_color', 'phone', 'email', 'locale', '_flags', 'verified', 'mfa_enabled',
+                 'premium', 'premium_type', '_relationships', 'note', '__weakref__')
 
     def __init__(self, *, state, data):
         super().__init__(state=state, data=data)
         self._relationships = {}
+        self.note = Note(state, self.id, user=self)
 
     def __repr__(self):
         return '<ClientUser id={0.id} name={0.name!r} discriminator={0.discriminator!r}' \
@@ -347,14 +607,17 @@ class ClientUser(BaseUser):
 
     def _update(self, data):
         super()._update(data)
-        # There's actually an Optional[str] phone field as well but I won't use it
         self.verified = data.get('verified', False)
         self.email = data.get('email')
+        self.phone = data.get('phone')
         self.locale = data.get('locale')
         self._flags = data.get('flags', 0)
         self.mfa_enabled = data.get('mfa_enabled', False)
         self.premium = data.get('premium', False)
         self.premium_type = try_enum(PremiumType, data.get('premium_type', None))
+        self.bio = data.get('bio')
+        self.banner = data.get('banner')
+        self._accent_color = data.get('accent_color')
 
     def get_relationship(self, user_id):
         """Retrieves the :class:`Relationship` if applicable.
@@ -404,22 +667,30 @@ class ClientUser(BaseUser):
         -----------
         password: :class:`str`
             The current password for the client's account.
-            Only applicable to user accounts.
+            Required for everything except avatar, banner, accent_colour, and bio.
         new_password: :class:`str`
             The new password you wish to change to.
-            Only applicable to user accounts.
         email: :class:`str`
             The new email you wish to change to.
-            Only applicable to user accounts.
         house: Optional[:class:`HypeSquadHouse`]
             The hypesquad house you wish to change to.
             Could be ``None`` to leave the current house.
-            Only applicable to user accounts.
         username: :class:`str`
             The new username you wish to change to.
+        discriminator: :class:`int`
+            The new discriminator you wish to change to.
+            Can only be used if you have Nitro.
         avatar: :class:`bytes`
             A :term:`py:bytes-like object` representing the image to upload.
             Could be ``None`` to denote no avatar.
+        banner: :class:`bytes`
+            A :term:`py:bytes-like object` representing the image to upload.
+            Could be ``None`` to denote no banner.
+        accent_colour/_color: :class:`Colour`
+            A :class:`Colour` object of the colour you want to set your profile to.
+        bio: :class:`str`
+            Your 'about me' section.
+            Could be ``None`` to represent no 'about me'.
 
         Raises
         ------
@@ -428,36 +699,60 @@ class ClientUser(BaseUser):
         InvalidArgument
             Wrong image format passed for ``avatar``.
         ClientException
-            Password was not passed.
+            Password was not passed when it was required.
             House field was not a HypeSquadHouse.
         """
 
-        try:
+        args = {}
+
+        if any(option in fields for option in ('new_password', 'email', 'username', 'discriminator')):
+            password = fields.get('password')
+            if password is None:
+                raise ClientException('Password is required')
+            args['password'] = password
+
+        if 'avatar' in fields:
             avatar_bytes = fields['avatar']
-        except KeyError:
-            avatar = self.avatar
-        else:
             if avatar_bytes is not None:
-                avatar = _bytes_to_base64_data(avatar_bytes)
+                args['avatar'] = _bytes_to_base64_data(avatar_bytes)
             else:
-                avatar = None
+                args['avatar'] = None
 
-        not_bot_account = not self.bot
-        password = fields.get('password')
-        if not_bot_account and password is None:
-            raise ClientException('Password is required')
+        if 'banner' in fields:
+            banner_bytes = fields['banner']
+            if banner_bytes is not None:
+                args['banner'] = _bytes_to_base64_data(banner_bytes)
+            else:
+                args['banner'] = None
 
-        args = {
-            'password': password,
-            'username': fields.get('username', self.name),
-            'avatar': avatar
-        }
+        if 'accent_color' or 'accent_colour' in fields:
+            accent_color = fields.get('accent_color', None)
+            accent_color = fields.get('accent_colour', accent_color)
+            if accent_color is None:
+                args['accent_color'] = accent_color
+            elif not isinstance(accent_color, Colour):
+                raise ClientException('`accent_colour` parameter was not a Colour')
+            else:
+                args['accent_color'] = accent_color.value
 
-        if not_bot_account:
-            args['email'] = fields.get('email', self.email)
+        if 'email' in fields:
+            args['email'] = fields['email']
 
-            if 'new_password' in fields:
-                args['new_password'] = fields['new_password']
+        if 'username' in fields:
+            args['username'] = fields['username']
+
+        if 'discriminator' in fields:
+            args['discriminator'] = fields['discriminator']
+
+        if 'new_password' in fields:
+            args['new_password'] = fields['new_password']
+
+        if 'bio' in fields:
+            bio = fields['bio']
+            if bio is not None:
+                args['bio'] = bio
+            else:
+                args['bio'] = ''
 
         http = self._state.http
 
@@ -473,12 +768,10 @@ class ClientUser(BaseUser):
             await http.change_hypesquad_house(value)
 
         data = await http.edit_profile(**args)
-        if not_bot_account:
-            self.email = data['email']
-            try:
-                http._token(data['token'], bot=False)
-            except KeyError:
-                pass
+        try:
+            http._token(data['token'])
+        except KeyError:
+            pass
 
         self._update(data)
 
@@ -515,7 +808,7 @@ class ClientUser(BaseUser):
             raise ClientException('You must have two or more recipients to create a group.')
 
         users = [str(u.id) for u in recipients]
-        data = await self._state.http.start_group(self.id, users)
+        data = await self._state.http.start_group(users)
         return GroupChannel(me=self, data=data, state=self._state)
 
     async def edit_settings(self, **kwargs):
@@ -524,12 +817,18 @@ class ClientUser(BaseUser):
         Edits the client user's settings.
 
         Parameters
-        -------
+        ----------
         afk_timeout: :class:`int`
             How long (in seconds) the user needs to be AFK until Discord
             sends push notifications to your mobile device.
+        allow_accessibility_detection: :class:`bool`
+            Whether or not to allow Discord to track screen reader usage.
         animate_emojis: :class:`bool`
             Whether or not to animate emojis in the chat.
+        animate_stickers: :class:`StickerAnimationOptions`
+            Whether or not to animate stickers in the chat.
+        contact_sync_enabled: :class:`bool`
+            Whether or not to enable the contact sync on Discord mobile.
         convert_emoticons: :class:`bool`
             Whether or not to automatically convert emoticons into emojis.
             e.g. :-) -> 😃
@@ -563,6 +862,9 @@ class ClientUser(BaseUser):
             of the Discord client.
         message_display_compact: :class:`bool`
             Whether or not to use the compact Discord display mode.
+        native_phone_integration_enabled: :class:`bool`
+            Whether or not to enable the new Discord mobile phone number friend
+            requesting features.
         render_embeds: :class:`bool`
             Whether or not to render embeds that are sent in the chat.
         render_reactions: :class:`bool`
@@ -571,23 +873,23 @@ class ClientUser(BaseUser):
             A list of guilds that you will not receive DMs from.
         show_current_game: :class:`bool`
             Whether or not to display the game that you are currently playing.
-        status: :class:`Status`
-            The clients status that is shown to others.
+        stream_notifications_enabled: :class:`bool`
+            Unknown.
         theme: :class:`Theme`
             The theme of the Discord UI.
         timezone_offset: :class:`int`
             The timezone offset to use.
+        view_nsfw_guilds: :class:`bool`
+            Whether or not to show NSFW guilds on iOS.
 
         Raises
         -------
         HTTPException
             Editing the settings failed.
-        Forbidden
-            The client is a bot user and not a user account.
 
         Returns
         -------
-        :class:`dict`
+        :class:`.Settings`
             The client user's updated settings.
         """
         payload = {}
@@ -596,11 +898,13 @@ class ClientUser(BaseUser):
         if content_filter:
             payload.update({'explicit_content_filter': content_filter.value})
 
+        animate_stickers = kwargs.pop('animate_stickers', None)
+        if animate_stickers:
+            payload.update({'animate_stickers': animate_stickers.value})
+
         friend_flags = kwargs.pop('friend_source_flags', None)
         if friend_flags:
-            dicts = [{}, {'mutual_guilds': True}, {'mutual_friends': True},
-            {'mutual_guilds': True, 'mutual_friends': True}, {'all': True}]
-            payload.update({'friend_source_flags': dicts[friend_flags.value]})
+            payload.update({'friend_source_flags': friend_flags.to_dict()})
 
         guild_positions = kwargs.pop('guild_positions', None)
         if guild_positions:
@@ -623,7 +927,106 @@ class ClientUser(BaseUser):
         payload.update(kwargs)
 
         data = await self._state.http.edit_settings(**payload)
-        return data
+        self.settings = settings = Settings(data=data, state=self._state)
+        return settings
+
+    @property
+    def banner_url(self):
+        """:class:`Asset`: Returns an :class:`Asset` for the banner the user has.
+
+        This is equivalent to calling :meth:`banner_url_as` with
+        the default parameters (i.e. webp/gif detection and a size of 1024).
+        """
+        return self.banner_url_as(format=None, size=1024)
+
+    def is_banner_animated(self):
+        """:class:`bool`: Indicates if the user has an animated banner."""
+        return bool(self.banner and self.banner.startswith('a_'))
+
+    def banner_url_as(self, *, format=None, static_format='webp', size=1024):
+        """Returns an :class:`Asset` for the banner the user has.
+
+        The format must be one of 'webp', 'jpeg', 'jpg', 'png' or 'gif', and
+        'gif' is only valid for animated banner. The size must be a power of
+        2 between 16 and 4096.
+
+        Parameters
+        -----------
+        format: Optional[:class:`str`]
+            The format to attempt to convert the avatar to.
+            If the format is ``None``, then it is automatically
+            detected into either 'gif' or static_format depending on the
+            banner being animated or not.
+        static_format: Optional[:class:`str`]
+            Format to attempt to convert only non-animated banners to.
+            Defaults to 'webp'
+        size: :class:`int`
+            The size of the image to display.
+
+        Raises
+        ------
+        InvalidArgument
+            Bad image format passed to ``format`` or ``static_format``, or
+            invalid ``size``.
+
+        Returns
+        --------
+        :class:`Asset`
+            The resulting CDN asset.
+        """
+        return Asset._from_user_banner(self._state, self, format=format, static_format=static_format, size=size, user=True)
+
+    @property
+    def accent_colour(self):
+        """:class:`Colour`: Returns the user's accent colour.
+
+        There is an alias for this named :attr:`accent_color`.
+
+        .. versionadded:: 1.9
+        """
+        if self._accent_color is None:
+            return None
+        return Colour(self._accent_color)
+
+    accent_color = accent_colour
+
+    def disable(self, password):
+        """|coro|
+
+        Disables the client's account.
+
+        .. versionadded:: 1.9
+
+        Parameters
+        -----------
+        password :class:`str`
+            The current password of the user.
+
+        Raises
+        -------
+        HTTPException
+            Disabling the account failed.
+        """
+        return self._state.http.disable_account(password)
+
+    def delete(self, password):
+        """|coro|
+
+        Deletes the client's account.
+
+        .. versionadded:: 1.9
+
+        Parameters
+        -----------
+        password :class:`str`
+            The current password of the user.
+
+        Raises
+        -------
+        HTTPException
+            Deleting the account failed.
+        """
+        return self._state.http.delete_account(password)
 
 class User(BaseUser, discord.abc.Messageable):
     """Represents a Discord user.
@@ -653,7 +1056,7 @@ class User(BaseUser, discord.abc.Messageable):
     id: :class:`int`
         The user's unique ID.
     discriminator: :class:`str`
-        The user's discriminator. This is given when the username has conflicts.
+        The user's discriminator.
     avatar: Optional[:class:`str`]
         The avatar hash the user has. Could be None.
     bot: :class:`bool`
@@ -680,18 +1083,6 @@ class User(BaseUser, discord.abc.Messageable):
         """
         return self._state._get_private_channel_by_user(self.id)
 
-    @property
-    def mutual_guilds(self):
-        """List[:class:`Guild`]: The guilds that the user shares with the client.
-
-        .. note::
-
-            This will only return mutual guilds within the client's internal cache.
-
-        .. versionadded:: 1.7
-        """
-        return [guild for guild in self._state._guilds.values() if guild.get_member(self.id)]
-
     async def create_dm(self):
         """|coro|
 
@@ -737,7 +1128,27 @@ class User(BaseUser, discord.abc.Messageable):
         """
         state = self._state
         mutuals = await state.http.get_mutual_friends(self.id)
-        return [User(state=state, data=friend) for friend in mutuals]
+        return [state.store_user(friend) for friend in mutuals]
+
+    async def mutual_guilds(self):
+        """|coro|
+
+        Gets all mutual guilds with this user.
+
+        Raises
+        -------
+        Forbidden
+            Not allowed to get mutual guilds of this user.
+        HTTPException
+            Getting mutual guilds failed.
+
+        Returns
+        -------
+        List[:class:`User`]
+            The users that are mutual friends.
+        """
+        profile = await self.profile()
+        return profile.mutual_guilds
 
     def is_friend(self):
         """:class:`bool`: Checks if the user is your friend."""
@@ -765,8 +1176,7 @@ class User(BaseUser, discord.abc.Messageable):
         HTTPException
             Blocking the user failed.
         """
-
-        await self._state.http.add_relationship(self.id, type=RelationshipType.blocked.value)
+        await self._state.http.add_relationship(self.id, type=RelationshipType.blocked.value, action=RelationshipAction.block)
 
     async def unblock(self):
         """|coro|
@@ -780,7 +1190,7 @@ class User(BaseUser, discord.abc.Messageable):
         HTTPException
             Unblocking the user failed.
         """
-        await self._state.http.remove_relationship(self.id)
+        await self._state.http.remove_relationship(self.id, action=RelationshipAction.unblock)
 
     async def remove_friend(self):
         """|coro|
@@ -794,7 +1204,7 @@ class User(BaseUser, discord.abc.Messageable):
         HTTPException
             Removing the user as a friend failed.
         """
-        await self._state.http.remove_relationship(self.id)
+        await self._state.http.remove_relationship(self.id, action=RelationshipAction.unfriend)
 
     async def send_friend_request(self):
         """|coro|
@@ -808,7 +1218,7 @@ class User(BaseUser, discord.abc.Messageable):
         HTTPException
             Sending the friend request failed.
         """
-        await self._state.http.send_friend_request(username=self.name, discriminator=self.discriminator)
+        await self._state.http.send_friend_request(self.name, self.discriminator)
 
     async def profile(self):
         """|coro|
@@ -830,26 +1240,9 @@ class User(BaseUser, discord.abc.Messageable):
 
         state = self._state
         data = await state.http.get_user_profile(self.id)
+        data['mutual_friends'] = await state.http.get_mutual_friends(self.id)
 
-        def transform(d):
-            return state._get_guild(int(d['id']))
+        profile = Profile(state, data)
+        await profile.note.fetch()
 
-        since = data.get('premium_since')
-        mutual_guilds = list(filter(None, map(transform, data.get('mutual_guilds', []))))
-        return Profile(flags=data['user'].get('flags', 0),
-                       premium_since=parse_time(since),
-                       mutual_guilds=mutual_guilds,
-                       user=self,
-                       connected_accounts=data['connected_accounts'])
-
-
-class LazyUser(BaseUser):
-    def __init__(self, state, user_id):
-        fake_data = {
-            'username': None,
-            'id': user_id,
-            'discriminator': 0000,
-            'avatar': None,
-        }
-
-        super().__init__(state=state, data=fake_data)
+        return profile
